@@ -10,7 +10,7 @@ class BaseSignal(FrozenList):
 
     @asyncio.coroutine
     def _send(self, *args, **kwargs):
-        for receiver in self:
+        for receiver in self._items:
             res = receiver(*args, **kwargs)
             if asyncio.iscoroutine(res) or isinstance(res, asyncio.Future):
                 yield from res
@@ -40,14 +40,36 @@ class Signal(BaseSignal):
         """
         Sends data to all registered receivers.
         """
-        ordinal = None
-        debug = self._app._debug
-        if debug:
-            ordinal = self._pre.ordinal()
-            yield from self._pre.send(ordinal, self._name, *args, **kwargs)
-        yield from self._send(*args, **kwargs)
-        if debug:
-            yield from self._post.send(ordinal, self._name, *args, **kwargs)
+        if self._items:
+            ordinal = None
+            debug = self._app._debug
+            if debug:
+                ordinal = self._pre.ordinal()
+                yield from self._pre.send(
+                    ordinal, self._name, *args, **kwargs)
+            yield from self._send(*args, **kwargs)
+            if debug:
+                yield from self._post.send(
+                    ordinal, self._name, *args, **kwargs)
+
+
+class FuncSignal(BaseSignal):
+    """Callback-based signal implementation.
+
+    To connect a callback to a signal, use any list method.
+
+    Signals are fired using the :meth:`send` method, which takes named
+    arguments.
+    """
+
+    __slots__ = ()
+
+    def send(self, *args, **kwargs):
+        """
+        Sends data to all registered receivers.
+        """
+        for receiver in self._items:
+            receiver(*args, **kwargs)
 
 
 class DebugSignal(BaseSignal):
