@@ -1,20 +1,15 @@
 import asyncio
 import collections
-import functools
-import sys
 import traceback
 
 from . import helpers
 from .log import internal_logger
 
 __all__ = (
-    'EofStream', 'StreamReader', 'DataQueue', 'ChunksQueue',
+    'EMPTY_PAYLOAD', 'EofStream', 'StreamReader', 'DataQueue', 'ChunksQueue',
     'FlowControlStreamReader',
     'FlowControlDataQueue', 'FlowControlChunksQueue')
 
-PY_35 = sys.version_info >= (3, 5)
-PY_352 = sys.version_info >= (3, 5, 2)
-
 DEFAULT_LIMIT = 2 ** 16
 
 
@@ -22,7 +17,7 @@ class EofStream(Exception):
     """eof stream indication."""
 
 
-if PY_35:
+if helpers.PY_35:
     class AsyncStreamIterator:
 
         def __init__(self, read_func):
@@ -31,7 +26,7 @@ if PY_35:
         def __aiter__(self):
             return self
 
-        if not PY_352:  # pragma: no cover
+        if not helpers.PY_352:  # pragma: no cover
             __aiter__ = asyncio.coroutine(__aiter__)
 
         @asyncio.coroutine
@@ -47,11 +42,11 @@ if PY_35:
 
 class AsyncStreamReaderMixin:
 
-    if PY_35:
+    if helpers.PY_35:
         def __aiter__(self):
             return AsyncStreamIterator(self.readline)
 
-        if not PY_352:  # pragma: no cover
+        if not helpers.PY_352:  # pragma: no cover
             __aiter__ = asyncio.coroutine(__aiter__)
 
         def iter_chunked(self, n):
@@ -91,19 +86,20 @@ class StreamReader(AsyncStreamReaderMixin):
         if loop is None:
             loop = asyncio.get_event_loop()
         self._loop = loop
+        self._size = 0
         self._buffer = collections.deque()
-        self._buffer_size = 0
         self._buffer_offset = 0
         self._eof = False
         self._waiter = None
         self._eof_waiter = None
         self._exception = None
         self._timer = timer
+        self._eof_callbacks = []
 
     def __repr__(self):
-        info = ['StreamReader']
-        if self._buffer_size:
-            info.append('%d bytes' % self._buffer_size)
+        info = [self.__class__.__name__]
+        if self._size:
+            info.append('%d bytes' % self._size)
         if self._eof:
             info.append('eof')
         if self._limit != DEFAULT_LIMIT:
@@ -119,28 +115,52 @@ class StreamReader(AsyncStreamReaderMixin):
 
     def set_exception(self, exc):
         self._exception = exc
+        self._eof_callbacks.clear()
 
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
+            if not waiter.done():
                 waiter.set_exception(exc)
 
+        waiter = self._eof_waiter
+        if waiter is not None:
+            self._eof_waiter = None
+            if not waiter.done():
+                waiter.set_exception(exc)
+
+    def on_eof(self, callback):
+        if self._eof:
+            try:
+                callback()
+            except Exception:
+                internal_logger.exception('Exception in eof callback')
+        else:
+            self._eof_callbacks.append(callback)
+
     def feed_eof(self):
         self._eof = True
 
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
+            if not waiter.done():
                 waiter.set_result(True)
 
         waiter = self._eof_waiter
         if waiter is not None:
             self._eof_waiter = None
-            if not waiter.cancelled():
+            if not waiter.done():
                 waiter.set_result(True)
 
+        for cb in self._eof_callbacks:
+            try:
+                cb()
+            except Exception:
+                internal_logger.exception('Exception in eof callback')
+
+        self._eof_callbacks.clear()
+
     def is_eof(self):
         """Return True if  'feed_eof' was called."""
         return self._eof
@@ -170,8 +190,8 @@ class StreamReader(AsyncStreamReaderMixin):
         if self._buffer_offset:
             self._buffer[0] = self._buffer[0][self._buffer_offset:]
             self._buffer_offset = 0
+        self._size += len(data)
         self._buffer.appendleft(data)
-        self._buffer_size += len(data)
 
     def feed_data(self, data):
         assert not self._eof, 'feed_data after feed_eof'
@@ -179,14 +199,14 @@ class StreamReader(AsyncStreamReaderMixin):
         if not data:
             return
 
+        self._size += len(data)
         self._buffer.append(data)
-        self._buffer_size += len(data)
         self.total_bytes += len(data)
 
         waiter = self._waiter
         if waiter is not None:
             self._waiter = None
-            if not waiter.cancelled():
+            if not waiter.done():
                 waiter.set_result(False)
 
     @asyncio.coroutine
@@ -335,7 +355,7 @@ class StreamReader(AsyncStreamReaderMixin):
         else:
             data = self._buffer.popleft()
 
-        self._buffer_size -= len(data)
+        self._size -= len(data)
         return data
 
     def _read_nowait(self, n):
@@ -360,6 +380,12 @@ class EmptyStreamReader(AsyncStreamReaderMixin):
     def set_exception(self, exc):
         pass
 
+    def on_eof(self, callback):
+        try:
+            callback()
+        except Exception:
+            internal_logger.exception('Exception in eof callback')
+
     def feed_eof(self):
         pass
 
@@ -396,6 +422,9 @@ class EmptyStreamReader(AsyncStreamReaderMixin):
         return b''
 
 
+EMPTY_PAYLOAD = EmptyStreamReader()
+
+
 class DataQueue:
     """DataQueue is a general-purpose blocking queue with one reader."""
 
@@ -407,6 +436,9 @@ class DataQueue:
         self._size = 0
         self._buffer = collections.deque()
 
+    def __len__(self):
+        return len(self._buffer)
+
     def is_eof(self):
         return self._eof
 
@@ -417,6 +449,7 @@ class DataQueue:
         return self._exception
 
     def set_exception(self, exc):
+        self._eof = True
         self._exception = exc
 
         waiter = self._waiter
@@ -468,11 +501,11 @@ class DataQueue:
             else:
                 raise EofStream
 
-    if PY_35:
+    if helpers.PY_35:
         def __aiter__(self):
             return AsyncStreamIterator(self.read)
 
-        if not PY_352:  # pragma: no cover
+        if not helpers.PY_352:  # pragma: no cover
             __aiter__ = asyncio.coroutine(__aiter__)
 
 
@@ -489,99 +522,58 @@ class ChunksQueue(DataQueue):
     readany = read
 
 
-def maybe_resume(func):
-
-    if asyncio.iscoroutinefunction(func):
-        @asyncio.coroutine
-        @functools.wraps(func)
-        def wrapper(self, *args, **kw):
-            result = yield from func(self, *args, **kw)
-            self._check_buffer_size()
-            return result
-    else:
-        @functools.wraps(func)
-        def wrapper(self, *args, **kw):
-            result = func(self, *args, **kw)
-            self._check_buffer_size()
-            return result
-
-    return wrapper
-
-
 class FlowControlStreamReader(StreamReader):
 
-    def __init__(self, stream, limit=DEFAULT_LIMIT, *args, **kwargs):
+    def __init__(self, protocol, buffer_limit=DEFAULT_LIMIT, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        self._stream = stream
-        self._b_limit = limit * 2
-        self._allow_pause = False
-
-        # resume transport reading
-        if stream.paused:
-            try:
-                self._stream.transport.resume_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = False
-                self._allow_pause = True
-
-    def _check_buffer_size(self):
-        if self._stream.paused:
-            if self._buffer_size < self._b_limit:
-                try:
-                    self._stream.transport.resume_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = False
-        else:
-            if self._buffer_size > self._b_limit:
-                try:
-                    self._stream.transport.pause_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = True
+        self._protocol = protocol
+        self._b_limit = buffer_limit * 2
 
     def feed_data(self, data, size=0):
-        has_waiter = self._waiter is not None and not self._waiter.cancelled()
-
         super().feed_data(data)
 
-        if (self._allow_pause and not self._stream.paused and
-                not has_waiter and self._buffer_size > self._b_limit):
-            try:
-                self._stream.transport.pause_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = True
+        if self._size > self._b_limit and not self._protocol._reading_paused:
+            self._protocol.pause_reading()
 
-    @maybe_resume
     @asyncio.coroutine
     def read(self, n=-1):
-        return (yield from super().read(n))
+        try:
+            return (yield from super().read(n))
+        finally:
+            if self._size < self._b_limit and self._protocol._reading_paused:
+                self._protocol.resume_reading()
 
-    @maybe_resume
     @asyncio.coroutine
     def readline(self):
-        return (yield from super().readline())
+        try:
+            return (yield from super().readline())
+        finally:
+            if self._size < self._b_limit and self._protocol._reading_paused:
+                self._protocol.resume_reading()
 
-    @maybe_resume
     @asyncio.coroutine
     def readany(self):
-        return (yield from super().readany())
+        try:
+            return (yield from super().readany())
+        finally:
+            if self._size < self._b_limit and self._protocol._reading_paused:
+                self._protocol.resume_reading()
 
-    @maybe_resume
     @asyncio.coroutine
     def readexactly(self, n):
-        return (yield from super().readexactly(n))
+        try:
+            return (yield from super().readexactly(n))
+        finally:
+            if self._size < self._b_limit and self._protocol._reading_paused:
+                self._protocol.resume_reading()
 
-    @maybe_resume
     def read_nowait(self, n=-1):
-        return super().read_nowait(n)
+        try:
+            return super().read_nowait(n)
+        finally:
+            if self._size < self._b_limit and self._protocol._reading_paused:
+                self._protocol.resume_reading()
 
 
 class FlowControlDataQueue(DataQueue):
@@ -589,59 +581,25 @@ class FlowControlDataQueue(DataQueue):
 
     It is a destination for parsed data."""
 
-    def __init__(self, stream, *, limit=DEFAULT_LIMIT, loop=None):
+    def __init__(self, protocol, *, limit=DEFAULT_LIMIT, loop=None):
         super().__init__(loop=loop)
 
-        self._stream = stream
+        self._protocol = protocol
         self._limit = limit * 2
-        self._allow_pause = False
-
-        # resume transport reading
-        if stream.paused:
-            try:
-                self._stream.transport.resume_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = False
-                self._allow_pause = True
 
     def feed_data(self, data, size):
-        has_waiter = self._waiter is not None and not self._waiter.cancelled()
-
         super().feed_data(data, size)
 
-        if (self._allow_pause and not self._stream.paused and
-                not has_waiter and self._size > self._limit):
-            try:
-                self._stream.transport.pause_reading()
-            except (AttributeError, NotImplementedError):
-                pass
-            else:
-                self._stream.paused = True
+        if self._size > self._limit and not self._protocol._reading_paused:
+            self._protocol.pause_reading()
 
     @asyncio.coroutine
     def read(self):
-        result = yield from super().read()
-
-        if self._stream.paused:
-            if self._size < self._limit:
-                try:
-                    self._stream.transport.resume_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = False
-        else:
-            if self._size > self._limit:
-                try:
-                    self._stream.transport.pause_reading()
-                except (AttributeError, NotImplementedError):
-                    pass
-                else:
-                    self._stream.paused = True
-
-        return result
+        try:
+            return (yield from super().read())
+        finally:
+            if self._size < self._limit and self._protocol._reading_paused:
+                self._protocol.resume_reading()
 
 
 class FlowControlChunksQueue(FlowControlDataQueue):
