@@ -1,23 +1,19 @@
 import asyncio
 import json
-import sys
-import warnings
 from collections import namedtuple
 
 from . import hdrs
-from ._ws_impl import (CLOSED_MESSAGE, CLOSING_MESSAGE, WebSocketError,
-                       WSMessage, WSMsgType, do_handshake)
-from .errors import ClientDisconnectedError, HttpProcessingError
-from .helpers import create_future
+from .helpers import PY_35, PY_352, Timeout, call_later, create_future
+from .http import (WS_CLOSED_MESSAGE, WS_CLOSING_MESSAGE, HttpProcessingError,
+                   WebSocketError, WebSocketReader,
+                   WSMessage, WSMsgType, do_handshake)
+from .streams import FlowControlDataQueue
 from .web_exceptions import (HTTPBadRequest, HTTPInternalServerError,
                              HTTPMethodNotAllowed)
-from .web_reqrep import StreamResponse
+from .web_response import StreamResponse
 
 __all__ = ('WebSocketResponse', 'WebSocketReady', 'MsgType', 'WSMsgType',)
 
-PY_35 = sys.version_info >= (3, 5)
-PY_352 = sys.version_info >= (3, 5, 2)
-
 THRESHOLD_CONNLOST_ACCESS = 5
 
 
@@ -38,7 +34,7 @@ class WebSocketResponse(StreamResponse):
                  protocols=()):
         super().__init__(status=101)
         self._protocols = protocols
-        self._protocol = None
+        self._ws_protocol = None
         self._writer = None
         self._reader = None
         self._closed = False
@@ -54,8 +50,9 @@ class WebSocketResponse(StreamResponse):
         self._autoping = autoping
         self._heartbeat = heartbeat
         self._heartbeat_cb = None
+        if heartbeat is not None:
+            self._pong_heartbeat = heartbeat/2.0
         self._pong_response_cb = None
-        self._time_service = None
 
     def _cancel_heartbeat(self):
         if self._pong_response_cb is not None:
@@ -70,8 +67,8 @@ class WebSocketResponse(StreamResponse):
         self._cancel_heartbeat()
 
         if self._heartbeat is not None:
-            self._heartbeat_cb = self._time_service.call_later(
-                self._heartbeat, self._send_heartbeat)
+            self._heartbeat_cb = call_later(
+                self._send_heartbeat, self._heartbeat, self._loop)
 
     def _send_heartbeat(self):
         if self._heartbeat is not None and not self._closed:
@@ -79,8 +76,8 @@ class WebSocketResponse(StreamResponse):
 
             if self._pong_response_cb is not None:
                 self._pong_response_cb.cancel()
-            self._pong_response_cb = self._time_service.call_later(
-                self._heartbeat/2.0, self._pong_not_received)
+            self._pong_response_cb = call_later(
+                self._pong_not_received, self._pong_heartbeat, self._loop)
 
     def _pong_not_received(self):
         self._closed = True
@@ -93,19 +90,21 @@ class WebSocketResponse(StreamResponse):
     @asyncio.coroutine
     def prepare(self, request):
         # make pre-check to don't hide it by do_handshake() exceptions
-        resp_impl = self._start_pre_check(request)
-        if resp_impl is not None:
-            return resp_impl
+        if self._payload_writer is not None:
+            return self._payload_writer
 
-        parser, protocol, writer = self._pre_start(request)
-        resp_impl = yield from super().prepare(request)
-        self._post_start(request, parser, protocol, writer)
-        return resp_impl
+        protocol, writer = self._pre_start(request)
+        payload_writer = yield from super().prepare(request)
+        self._post_start(request, protocol, writer)
+        yield from payload_writer.drain()
+        return payload_writer
 
     def _pre_start(self, request):
+        self._loop = request.app.loop
+
         try:
-            status, headers, parser, writer, protocol = do_handshake(
-                request.method, request.headers, request.transport_pair[1],
+            status, headers, _, writer, protocol = do_handshake(
+                request.method, request.headers, request._protocol.writer,
                 self._protocols)
         except HttpProcessingError as err:
             if err.code == 405:
@@ -116,7 +115,6 @@ class WebSocketResponse(StreamResponse):
             else:  # pragma: no cover
                 raise HTTPInternalServerError() from err
 
-        self._time_service = request.time_service
         self._reset_heartbeat()
 
         if self.status != status:
@@ -124,42 +122,27 @@ class WebSocketResponse(StreamResponse):
         for k, v in headers:
             self.headers[k] = v
         self.force_close()
-        return parser, protocol, writer
+        return protocol, writer
 
-    def _post_start(self, request, parser, protocol, writer):
-        self._reader = request._reader.set_parser(parser)
+    def _post_start(self, request, protocol, writer):
+        self._ws_protocol = protocol
         self._writer = writer
-        self._protocol = protocol
-        self._loop = request.app.loop
-
-    def start(self, request):
-        warnings.warn('use .prepare(request) instead', DeprecationWarning)
-        # make pre-check to don't hide it by do_handshake() exceptions
-        resp_impl = self._start_pre_check(request)
-        if resp_impl is not None:
-            return resp_impl
-
-        parser, protocol, writer = self._pre_start(request)
-        resp_impl = super().start(request)
-        self._post_start(request, parser, protocol, writer)
-        return resp_impl
+        self._reader = FlowControlDataQueue(
+            request._protocol, limit=2 ** 16, loop=self._loop)
+        request.protocol.set_parser(WebSocketReader(self._reader))
 
     def can_prepare(self, request):
         if self._writer is not None:
             raise RuntimeError('Already started')
         try:
             _, _, _, _, protocol = do_handshake(
-                request.method, request.headers, request.transport,
+                request.method, request.headers, request._protocol.writer,
                 self._protocols)
         except HttpProcessingError:
             return WebSocketReady(False, None)
         else:
             return WebSocketReady(True, protocol)
 
-    def can_start(self, request):
-        warnings.warn('use .can_prepare(request) instead', DeprecationWarning)
-        return self.can_prepare(request)
-
     @property
     def closed(self):
         return self._closed
@@ -169,8 +152,8 @@ class WebSocketResponse(StreamResponse):
         return self._close_code
 
     @property
-    def protocol(self):
-        return self._protocol
+    def ws_protocol(self):
+        return self._ws_protocol
 
     def exception(self):
         return self._exception
@@ -208,7 +191,7 @@ class WebSocketResponse(StreamResponse):
     def write_eof(self):
         if self._eof_sent:
             return
-        if self._resp_impl is None:
+        if self._payload_writer is None:
             raise RuntimeError("Response has not been started")
 
         yield from self.close()
@@ -222,7 +205,7 @@ class WebSocketResponse(StreamResponse):
         # we need to break `receive()` cycle first,
         # `close()` may be called from different task
         if self._waiting is not None and not self._closed:
-            self._reader.feed_data(CLOSING_MESSAGE, 0)
+            self._reader.feed_data(WS_CLOSING_MESSAGE, 0)
             yield from self._waiting
 
         if not self._closed:
@@ -243,7 +226,7 @@ class WebSocketResponse(StreamResponse):
                 return True
 
             try:
-                with self._time_service.timeout(self._timeout):
+                with Timeout(self._timeout, loop=self._loop):
                     msg = yield from self._reader.read()
             except asyncio.CancelledError:
                 self._close_code = 1006
@@ -277,31 +260,28 @@ class WebSocketResponse(StreamResponse):
                 self._conn_lost += 1
                 if self._conn_lost >= THRESHOLD_CONNLOST_ACCESS:
                     raise RuntimeError('WebSocket connection is closed.')
-                return CLOSED_MESSAGE
+                return WS_CLOSED_MESSAGE
             elif self._closing:
-                return CLOSING_MESSAGE
+                return WS_CLOSING_MESSAGE
 
             try:
                 self._waiting = create_future(self._loop)
                 try:
-                    with self._time_service.timeout(
-                            timeout or self._receive_timeout):
+                    with Timeout(
+                            timeout or self._receive_timeout, loop=self._loop):
                         msg = yield from self._reader.read()
-                        self._reset_heartbeat()
                 finally:
+                    self._reset_heartbeat()
                     waiter = self._waiting
                     self._waiting = None
                     waiter.set_result(True)
             except (asyncio.CancelledError, asyncio.TimeoutError) as exc:
+                self._close_code = 1006
                 raise
             except WebSocketError as exc:
                 self._close_code = exc.code
                 yield from self.close(code=exc.code)
                 return WSMessage(WSMsgType.ERROR, exc, None)
-            except ClientDisconnectedError:
-                self._closed = True
-                self._close_code = 1006
-                return CLOSED_MESSAGE
             except Exception as exc:
                 self._exception = exc
                 self._closing = True
@@ -324,20 +304,13 @@ class WebSocketResponse(StreamResponse):
 
             return msg
 
-    @asyncio.coroutine
-    def receive_msg(self):
-        warnings.warn(
-            'receive_msg() coroutine is deprecated. use receive() instead',
-            DeprecationWarning)
-        return (yield from self.receive())
-
     @asyncio.coroutine
     def receive_str(self, *, timeout=None):
         msg = yield from self.receive(timeout)
         if msg.type != WSMsgType.TEXT:
             raise TypeError(
-                "Received message {}:{!r} is not str".format(msg.type,
-                                                             msg.data))
+                "Received message {}:{!r} is not WSMsgType.TEXT".format(
+                    msg.type, msg.data))
         return msg.data
 
     @asyncio.coroutine
