@@ -48,7 +48,7 @@ async def on_shutdown(app):
 
 
 async def init(loop):
-    app = Application(loop=loop)
+    app = Application()
     app['sockets'] = []
     app.router.add_get('/', wshandler)
     app.on_shutdown.append(on_shutdown)
@@ -57,4 +57,4 @@ async def init(loop):
 
 loop = asyncio.get_event_loop()
 app = loop.run_until_complete(init(loop))
-run_app(app, loop=loop)
+run_app(app)
