@@ -4,6 +4,8 @@ import socket
 import unittest
 from unittest import mock
 
+from yarl import URL
+
 import aiohttp
 from aiohttp.client_reqrep import ClientRequest, ClientResponse
 from aiohttp.test_utils import make_mocked_coro
@@ -25,25 +27,26 @@ class TestProxy(unittest.TestCase):
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_connect(self, ClientRequestMock):
         req = ClientRequest(
-            'GET', 'http://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('http://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop
         )
-        self.assertEqual(req.proxy, 'http://proxy.example.com')
+        self.assertEqual(str(req.proxy), 'http://proxy.example.com')
 
         # mock all the things!
         connector = aiohttp.TCPConnector(loop=self.loop)
         connector._resolve_host = make_mocked_coro([mock.MagicMock()])
 
-        tr, proto = mock.Mock(), mock.Mock()
-        self.loop.create_connection = make_mocked_coro((tr, proto))
+        proto = mock.Mock()
+        self.loop.create_connection = make_mocked_coro(
+            (proto.transport, proto))
         conn = self.loop.run_until_complete(connector.connect(req))
-        self.assertEqual(req.path, 'http://www.python.org/')
-        self.assertIs(conn._transport, tr)
+        self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertIs(conn._protocol, proto)
+        self.assertIs(conn.transport, proto.transport)
 
         ClientRequestMock.assert_called_with(
-            'GET', 'http://proxy.example.com',
+            'GET', URL('http://proxy.example.com'),
             auth=None,
             headers={'Host': 'www.python.org'},
             loop=self.loop)
@@ -51,8 +54,8 @@ class TestProxy(unittest.TestCase):
     def test_proxy_auth(self):
         with self.assertRaises(ValueError) as ctx:
             ClientRequest(
-                'GET', 'http://python.org',
-                proxy='http://proxy.example.com',
+                'GET', URL('http://python.org'),
+                proxy=URL('http://proxy.example.com'),
                 proxy_auth=('user', 'pass'),
                 loop=mock.Mock())
         self.assertEqual(
@@ -60,26 +63,41 @@ class TestProxy(unittest.TestCase):
             "proxy_auth must be None or BasicAuth() tuple",
         )
 
+    @mock.patch('aiohttp.client_reqrep.PayloadWriter')
+    def _test_connect_request_with_unicode_host(self, Request_mock):
+        loop = mock.Mock()
+        request = ClientRequest("CONNECT", URL("http://éé.com/"),
+                                loop=loop)
+
+        request.response_class = mock.Mock()
+        request.write_bytes = mock.Mock()
+        request.write_bytes.return_value = asyncio.Future(loop=loop)
+        request.write_bytes.return_value.set_result(None)
+        request.send(mock.Mock())
+
+        Request_mock.assert_called_with(mock.ANY, mock.ANY, "xn--9caa.com:80",
+                                        mock.ANY, loop=loop)
+
     def test_proxy_connection_error(self):
         connector = aiohttp.TCPConnector(loop=self.loop)
         connector._resolve_host = make_mocked_coro(
             raise_exception=OSError('dont take it serious'))
 
         req = ClientRequest(
-            'GET', 'http://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('http://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
         expected_headers = dict(req.headers)
-        with self.assertRaises(aiohttp.ProxyConnectionError):
+        with self.assertRaises(aiohttp.ClientConnectorError):
             self.loop.run_until_complete(connector.connect(req))
-        self.assertEqual(req.path, '/')
+        self.assertEqual(req.url.path, '/')
         self.assertEqual(dict(req.headers), expected_headers)
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_auth(self, ClientRequestMock):
         proxy_req = ClientRequest(
-            'GET', 'http://proxy.example.com',
+            'GET', URL('http://proxy.example.com'),
             auth=aiohttp.helpers.BasicAuth('user', 'pass'),
             loop=self.loop
         )
@@ -94,8 +112,8 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'http://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('http://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             proxy_auth=aiohttp.helpers.BasicAuth('user', 'pass'),
             loop=self.loop,
         )
@@ -103,28 +121,29 @@ class TestProxy(unittest.TestCase):
         self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
         conn = self.loop.run_until_complete(connector.connect(req))
 
-        self.assertEqual(req.path, 'http://www.python.org/')
+        self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertIn('PROXY-AUTHORIZATION', req.headers)
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
 
         ClientRequestMock.assert_called_with(
-            'GET', 'http://proxy.example.com',
+            'GET', URL('http://proxy.example.com'),
             auth=aiohttp.helpers.BasicAuth('user', 'pass'),
             loop=mock.ANY, headers=mock.ANY)
         conn.close()
 
     def test_auth_utf8(self):
         proxy_req = ClientRequest(
-            'GET', 'http://proxy.example.com',
+            'GET', URL('http://proxy.example.com'),
             auth=aiohttp.helpers.BasicAuth('юзер', 'пасс', 'utf-8'),
             loop=self.loop)
         self.assertIn('AUTHORIZATION', proxy_req.headers)
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_auth_from_url(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://user:pass@proxy.example.com',
+        proxy_req = ClientRequest('GET',
+                                  URL('http://user:pass@proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
         self.assertIn('AUTHORIZATION', proxy_req.headers)
@@ -137,28 +156,29 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'http://www.python.org',
-            proxy='http://user:pass@proxy.example.com',
+            'GET', URL('http://www.python.org'),
+            proxy=URL('http://user:pass@proxy.example.com'),
             loop=self.loop,
         )
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
         conn = self.loop.run_until_complete(connector.connect(req))
 
-        self.assertEqual(req.path, 'http://www.python.org/')
+        self.assertEqual(req.url, URL('http://www.python.org'))
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertIn('PROXY-AUTHORIZATION', req.headers)
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
 
         ClientRequestMock.assert_called_with(
-            'GET', 'http://user:pass@proxy.example.com',
+            'GET', URL('http://user:pass@proxy.example.com'),
             auth=None, loop=mock.ANY, headers=mock.ANY)
         conn.close()
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_auth__not_modifying_request(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://user:pass@proxy.example.com',
+        proxy_req = ClientRequest('GET',
+                                  URL('http://user:pass@proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
         proxy_req_headers = dict(proxy_req.headers)
@@ -168,24 +188,24 @@ class TestProxy(unittest.TestCase):
             raise_exception=OSError('nothing personal'))
 
         req = ClientRequest(
-            'GET', 'http://www.python.org',
-            proxy='http://user:pass@proxy.example.com',
+            'GET', URL('http://www.python.org'),
+            proxy=URL('http://user:pass@proxy.example.com'),
             loop=self.loop,
         )
         req_headers = dict(req.headers)
-        with self.assertRaises(aiohttp.ProxyConnectionError):
+        with self.assertRaises(aiohttp.ClientConnectorError):
             self.loop.run_until_complete(connector.connect(req))
         self.assertEqual(req.headers, req_headers)
-        self.assertEqual(req.path, '/')
+        self.assertEqual(req.url.path, '/')
         self.assertEqual(proxy_req.headers, proxy_req_headers)
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_connect(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', 'http://proxy.example.com')
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
         proxy_resp._loop = self.loop
         proxy_req.send = send_mock = mock.Mock()
         send_mock.return_value = proxy_resp
@@ -200,16 +220,16 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'https://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
         self.loop.run_until_complete(connector._create_connection(req))
 
-        self.assertEqual(req.path, '/')
+        self.assertEqual(req.url.path, '/')
         self.assertEqual(proxy_req.method, 'CONNECT')
-        self.assertEqual(proxy_req.path, 'www.python.org:443')
-        tr.pause_reading.assert_called_once_with()
+        self.assertEqual(proxy_req.url, URL('https://www.python.org'))
+        tr.close.assert_called_once_with()
         tr.get_extra_info.assert_called_with('socket', default=None)
 
         self.loop.run_until_complete(proxy_req.close())
@@ -218,11 +238,11 @@ class TestProxy(unittest.TestCase):
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_connect_runtime_error(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', 'http://proxy.example.com')
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
         proxy_resp._loop = self.loop
         proxy_req.send = send_mock = mock.Mock()
         send_mock.return_value = proxy_resp
@@ -238,8 +258,8 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'https://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
         with self.assertRaisesRegex(
@@ -252,11 +272,11 @@ class TestProxy(unittest.TestCase):
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_connect_http_proxy_error(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', 'http://proxy.example.com')
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
         proxy_resp._loop = self.loop
         proxy_req.send = send_mock = mock.Mock()
         send_mock.return_value = proxy_resp
@@ -274,12 +294,12 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'https://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
         with self.assertRaisesRegex(
-                aiohttp.HttpProxyError, "400, message='bad request'"):
+                aiohttp.ClientHttpProxyError, "400, message='bad request'"):
             self.loop.run_until_complete(connector._create_connection(req))
 
         self.loop.run_until_complete(proxy_req.close())
@@ -288,11 +308,11 @@ class TestProxy(unittest.TestCase):
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_connect_resp_start_error(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', 'http://proxy.example.com')
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
         proxy_resp._loop = self.loop
         proxy_req.send = send_mock = mock.Mock()
         send_mock.return_value = proxy_resp
@@ -309,8 +329,8 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'https://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
         with self.assertRaisesRegex(OSError, "error message"):
@@ -318,7 +338,7 @@ class TestProxy(unittest.TestCase):
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_request_port(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
@@ -332,35 +352,35 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'http://localhost:1234/path',
-            proxy='http://proxy.example.com',
+            'GET', URL('http://localhost:1234/path'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
         self.loop.run_until_complete(connector._create_connection(req))
-        self.assertEqual(req.path, 'http://localhost:1234/path')
+        self.assertEqual(req.url, URL('http://localhost:1234/path'))
 
     def test_proxy_auth_property(self):
         req = aiohttp.ClientRequest(
-            'GET', 'http://localhost:1234/path',
-            proxy='http://proxy.example.com',
+            'GET', URL('http://localhost:1234/path'),
+            proxy=URL('http://proxy.example.com'),
             proxy_auth=aiohttp.helpers.BasicAuth('user', 'pass'),
             loop=self.loop)
         self.assertEqual(('user', 'pass', 'latin1'), req.proxy_auth)
 
     def test_proxy_auth_property_default(self):
         req = aiohttp.ClientRequest(
-            'GET', 'http://localhost:1234/path',
-            proxy='http://proxy.example.com',
+            'GET', URL('http://localhost:1234/path'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop)
         self.assertIsNone(req.proxy_auth)
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_connect_pass_ssl_context(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', 'http://proxy.example.com')
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
         proxy_resp._loop = self.loop
         proxy_req.send = send_mock = mock.Mock()
         send_mock.return_value = proxy_resp
@@ -375,8 +395,8 @@ class TestProxy(unittest.TestCase):
         self.loop.create_connection = make_mocked_coro((tr, proto))
 
         req = ClientRequest(
-            'GET', 'https://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop,
         )
         self.loop.run_until_complete(connector._create_connection(req))
@@ -387,10 +407,10 @@ class TestProxy(unittest.TestCase):
             sock=mock.ANY,
             server_hostname='www.python.org')
 
-        self.assertEqual(req.path, '/')
+        self.assertEqual(req.url.path, '/')
         self.assertEqual(proxy_req.method, 'CONNECT')
-        self.assertEqual(proxy_req.path, 'www.python.org:443')
-        tr.pause_reading.assert_called_once_with()
+        self.assertEqual(proxy_req.url, URL('https://www.python.org'))
+        tr.close.assert_called_once_with()
         tr.get_extra_info.assert_called_with('socket', default=None)
 
         self.loop.run_until_complete(proxy_req.close())
@@ -399,13 +419,13 @@ class TestProxy(unittest.TestCase):
 
     @mock.patch('aiohttp.connector.ClientRequest')
     def test_https_auth(self, ClientRequestMock):
-        proxy_req = ClientRequest('GET', 'http://proxy.example.com',
+        proxy_req = ClientRequest('GET', URL('http://proxy.example.com'),
                                   auth=aiohttp.helpers.BasicAuth('user',
                                                                  'pass'),
                                   loop=self.loop)
         ClientRequestMock.return_value = proxy_req
 
-        proxy_resp = ClientResponse('get', 'http://proxy.example.com')
+        proxy_resp = ClientResponse('get', URL('http://proxy.example.com'))
         proxy_resp._loop = self.loop
         proxy_req.send = send_mock = mock.Mock()
         send_mock.return_value = proxy_resp
@@ -423,15 +443,15 @@ class TestProxy(unittest.TestCase):
         self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
 
         req = ClientRequest(
-            'GET', 'https://www.python.org',
-            proxy='http://proxy.example.com',
+            'GET', URL('https://www.python.org'),
+            proxy=URL('http://proxy.example.com'),
             loop=self.loop
         )
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
         self.loop.run_until_complete(connector._create_connection(req))
 
-        self.assertEqual(req.path, '/')
+        self.assertEqual(req.url.path, '/')
         self.assertNotIn('AUTHORIZATION', req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', req.headers)
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
@@ -442,63 +462,3 @@ class TestProxy(unittest.TestCase):
         self.loop.run_until_complete(proxy_req.close())
         proxy_resp.close()
         self.loop.run_until_complete(req.close())
-
-
-class TestProxyConnector(unittest.TestCase):
-
-    """
-    Backward compatibility simple test. Most testing happens in TextProxy.
-    """
-
-    def setUp(self):
-        self.loop = asyncio.new_event_loop()
-        asyncio.set_event_loop(None)
-
-    def tearDown(self):
-        # just in case if we have transport close callbacks
-        self.loop.stop()
-        self.loop.run_forever()
-        self.loop.close()
-        gc.collect()
-
-    def test_ctor(self):
-        connector = aiohttp.ProxyConnector(
-            'http://localhost:8118',
-            proxy_auth=aiohttp.helpers.BasicAuth('user', 'pass'),
-            loop=self.loop,
-        )
-
-        self.assertEqual('http://localhost:8118', connector.proxy)
-        self.assertEqual(
-            aiohttp.helpers.BasicAuth('user', 'pass'),
-            connector.proxy_auth
-        )
-        self.assertTrue(connector.force_close)
-
-    @mock.patch('aiohttp.connector.ClientRequest')
-    def test_connect(self, ClientRequestMock):
-        req = ClientRequest('GET', 'http://www.python.org', loop=self.loop)
-        self.assertEqual(req.path, '/')
-
-        connector = aiohttp.ProxyConnector('http://proxy.example.com',
-                                           loop=self.loop)
-        self.assertIs(self.loop, connector._loop)
-
-        connector._resolve_host = make_mocked_coro([mock.MagicMock()])
-
-        tr, proto = mock.Mock(), mock.Mock()
-        self.loop.create_connection = make_mocked_coro((tr, proto))
-        conn = self.loop.run_until_complete(connector.connect(req))
-        self.assertEqual(req.path, 'http://www.python.org/')
-        self.assertIs(conn._transport, tr)
-        self.assertIs(conn._protocol, proto)
-
-        # resolve_host.assert_called_once_with('proxy.example.com', 80)
-        tr.get_extra_info.assert_called_once_with('sslcontext')
-
-        ClientRequestMock.assert_called_with(
-            'GET', 'http://proxy.example.com',
-            auth=None,
-            headers={'Host': 'www.python.org'},
-            loop=self.loop)
-        conn.close()
