@@ -11,16 +11,26 @@ from aiohttp.client import _RequestContextManager
 async def test_await(test_server, loop):
 
     async def handler(request):
-        return web.HTTPOk()
+        resp = web.StreamResponse(headers={'content-length': str(4)})
+        await resp.prepare(request)
+        await resp.drain()
+        await asyncio.sleep(0.01, loop=loop)
+        resp.write(b'test')
+        await asyncio.sleep(0.01, loop=loop)
+        await resp.write_eof()
+        return resp
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     server = await test_server(app)
-    resp = await aiohttp.get(server.make_url('/'), loop=loop)
-    assert resp.status == 200
-    assert resp.connection is not None
-    await resp.release()
-    assert resp.connection is None
+
+    with aiohttp.ClientSession(loop=loop) as session:
+        resp = await session.get(server.make_url('/'))
+        assert resp.status == 200
+        assert resp.connection is not None
+        await resp.read()
+        await resp.release()
+        assert resp.connection is None
 
 
 async def test_response_context_manager(test_server, loop):
@@ -28,13 +38,13 @@ async def test_response_context_manager(test_server, loop):
     async def handler(request):
         return web.HTTPOk()
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     server = await test_server(app)
-    resp = await aiohttp.get(server.make_url('/'), loop=loop)
+    resp = await aiohttp.ClientSession(loop=loop).get(server.make_url('/'))
     async with resp:
         assert resp.status == 200
-        assert resp.connection is not None
+        assert resp.connection is None
     assert resp.connection is None
 
 
@@ -43,18 +53,20 @@ async def test_response_context_manager_error(test_server, loop):
     async def handler(request):
         return web.HTTPOk()
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     server = await test_server(app)
-    cm = aiohttp.get(server.make_url('/'), loop=loop)
-    session = cm._session
+    session = aiohttp.ClientSession(loop=loop)
+    cm = session.get(server.make_url('/'))
     resp = await cm
     with pytest.raises(RuntimeError):
         async with resp:
             assert resp.status == 200
             resp.content.set_exception(RuntimeError())
             await resp.read()
-    assert len(session._connector._conns) == 0
+            assert resp.closed
+
+    assert len(session._connector._conns) == 1
 
 
 async def test_client_api_context_manager(test_server, loop):
@@ -62,13 +74,14 @@ async def test_client_api_context_manager(test_server, loop):
     async def handler(request):
         return web.HTTPOk()
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     server = await test_server(app)
 
-    async with aiohttp.get(server.make_url('/'), loop=loop) as resp:
-        assert resp.status == 200
-        assert resp.connection is not None
+    async with aiohttp.ClientSession(loop=loop) as session:
+        async with session.get(server.make_url('/')) as resp:
+            assert resp.status == 200
+            assert resp.connection is None
     assert resp.connection is None
 
 
@@ -81,22 +94,23 @@ async def test_context_manager_close_on_release(test_server, loop, mocker):
     async def handler(request):
         resp = web.StreamResponse()
         await resp.prepare(request)
+        await resp.drain()
         await asyncio.sleep(10, loop=loop)
         return resp
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     server = await test_server(app)
 
     with aiohttp.ClientSession(loop=loop) as session:
         resp = await session.get(server.make_url('/'))
-        conn = resp.connection
-        mocker.spy(conn, 'close')
+        proto = resp.connection._protocol
+        mocker.spy(proto, 'close')
         async with resp:
             assert resp.status == 200
             assert resp.connection is not None
         assert resp.connection is None
-        assert conn.close.called
+        assert proto.close.called
 
 
 async def test_iter_any(test_server, loop):
@@ -110,7 +124,7 @@ async def test_iter_any(test_server, loop):
         assert b''.join(buf) == data
         return web.Response()
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('POST', '/', handler)
     server = await test_server(app)
 
