@@ -785,3 +785,147 @@ class TestChunksQueue(unittest.TestCase, DataQueueMixin):
 
     def test_readany(self):
         self.assertIs(self.buffer.read.__func__, self.buffer.readany.__func__)
+
+
+def test_feed_data_waiters(loop):
+    reader = streams.StreamReader(loop=loop)
+    waiter = reader._waiter = helpers.create_future(loop)
+    eof_waiter = reader._eof_waiter = helpers.create_future(loop)
+
+    reader.feed_data(b'1')
+    assert list(reader._buffer) == [b'1']
+    assert reader._size == 1
+    assert reader.total_bytes == 1
+
+    assert waiter.done()
+    assert not eof_waiter.done()
+    assert reader._waiter is None
+    assert reader._eof_waiter is eof_waiter
+
+
+def test_feed_data_completed_waiters(loop):
+    reader = streams.StreamReader(loop=loop)
+    waiter = reader._waiter = helpers.create_future(loop)
+
+    waiter.set_result(1)
+    reader.feed_data(b'1')
+
+    assert reader._waiter is None
+
+
+def test_feed_eof_waiters(loop):
+    reader = streams.StreamReader(loop=loop)
+    waiter = reader._waiter = helpers.create_future(loop)
+    eof_waiter = reader._eof_waiter = helpers.create_future(loop)
+
+    reader.feed_eof()
+    assert reader._eof
+
+    assert waiter.done()
+    assert eof_waiter.done()
+    assert reader._waiter is None
+    assert reader._eof_waiter is None
+
+
+def test_feed_eof_cancelled(loop):
+    reader = streams.StreamReader(loop=loop)
+    waiter = reader._waiter = helpers.create_future(loop)
+    eof_waiter = reader._eof_waiter = helpers.create_future(loop)
+
+    waiter.set_result(1)
+    eof_waiter.set_result(1)
+
+    reader.feed_eof()
+
+    assert waiter.done()
+    assert eof_waiter.done()
+    assert reader._waiter is None
+    assert reader._eof_waiter is None
+
+
+def test_on_eof(loop):
+    reader = streams.StreamReader(loop=loop)
+
+    on_eof = mock.Mock()
+    reader.on_eof(on_eof)
+
+    assert not on_eof.called
+    reader.feed_eof()
+    assert on_eof.called
+
+
+def test_on_eof_exc_in_callback(loop):
+    reader = streams.StreamReader(loop=loop)
+
+    on_eof = mock.Mock()
+    on_eof.side_effect = ValueError
+
+    reader.on_eof(on_eof)
+    assert not on_eof.called
+    reader.feed_eof()
+    assert on_eof.called
+    assert not reader._eof_callbacks
+
+
+def test_on_eof_eof_is_set(loop):
+    reader = streams.StreamReader(loop=loop)
+    reader.feed_eof()
+
+    on_eof = mock.Mock()
+    reader.on_eof(on_eof)
+    assert on_eof.called
+    assert not reader._eof_callbacks
+
+
+def test_on_eof_eof_is_set_exception(loop):
+    reader = streams.StreamReader(loop=loop)
+    reader.feed_eof()
+
+    on_eof = mock.Mock()
+    on_eof.side_effect = ValueError
+
+    reader.on_eof(on_eof)
+    assert on_eof.called
+    assert not reader._eof_callbacks
+
+
+def test_set_exception(loop):
+    reader = streams.StreamReader(loop=loop)
+    waiter = reader._waiter = helpers.create_future(loop)
+    eof_waiter = reader._eof_waiter = helpers.create_future(loop)
+
+    exc = ValueError()
+    reader.set_exception(exc)
+
+    assert waiter.exception() is exc
+    assert eof_waiter.exception() is exc
+    assert reader._waiter is None
+    assert reader._eof_waiter is None
+
+
+def test_set_exception_cancelled(loop):
+    reader = streams.StreamReader(loop=loop)
+    waiter = reader._waiter = helpers.create_future(loop)
+    eof_waiter = reader._eof_waiter = helpers.create_future(loop)
+
+    waiter.set_result(1)
+    eof_waiter.set_result(1)
+
+    exc = ValueError()
+    reader.set_exception(exc)
+
+    assert waiter.exception() is None
+    assert eof_waiter.exception() is None
+    assert reader._waiter is None
+    assert reader._eof_waiter is None
+
+
+def test_set_exception_eof_callbacks(loop):
+    reader = streams.StreamReader(loop=loop)
+
+    on_eof = mock.Mock()
+    reader.on_eof(on_eof)
+
+    reader.set_exception(ValueError())
+    assert not on_eof.called
+    assert not reader._eof_callbacks
