@@ -103,19 +103,38 @@ def test_head_returns_empty_body(loop, test_client):
 
 
 @asyncio.coroutine
-def test_post_form(loop, test_client):
+def test_response_before_complete(loop, test_client):
 
     @asyncio.coroutine
     def handler(request):
-        data = yield from request.post()
-        assert {'a': '1', 'b': '2'} == data
         return web.Response(body=b'OK')
 
     app = web.Application()
     app.router.add_post('/', handler)
     client = yield from test_client(app)
 
-    resp = yield from client.post('/', data={'a': 1, 'b': 2})
+    data = b'0' * 1024 * 1024
+
+    resp = yield from client.post('/', data=data)
+    assert 200 == resp.status
+    text = yield from resp.text()
+    assert 'OK' == text
+
+
+@asyncio.coroutine
+def test_post_form(loop, test_client):
+
+    @asyncio.coroutine
+    def handler(request):
+        data = yield from request.post()
+        assert {'a': '1', 'b': '2', 'c': ''} == data
+        return web.Response(body=b'OK')
+
+    app = web.Application()
+    app.router.add_post('/', handler)
+    client = yield from test_client(app)
+
+    resp = yield from client.post('/', data={'a': 1, 'b': 2, 'c': ''})
     assert 200 == resp.status
     txt = yield from resp.text()
     assert 'OK' == txt
